@@ -23,35 +23,42 @@
 #include <gnome.h>
 #include <gconf/gconf-client.h>
 #include <eel/eel-gconf-extensions.h>
+#include <libgnomevfs/gnome-vfs.h>
 
-#include "seahorse-key-manager.h"
+#include "seahorse-windows.h"
 #include "seahorse-widget.h"
-#include "seahorse-generate.h"
 #include "seahorse-preferences.h"
-#include "seahorse-import.h"
-#include "seahorse-text-editor.h"
 #include "seahorse-util.h"
 #include "seahorse-validity.h"
 #include "seahorse-key-manager-store.h"
-#include "seahorse-ops-key.h"
 #include "seahorse-key-dialogs.h"
-#include "seahorse-file-dialogs.h"
-#include "seahorse-key-pair.h"
+#include "seahorse-key-op.h"
+#include "seahorse-key-widget.h"
+#include "seahorse-op.h"
 
 #define KEY_LIST "key_list"
 
-#define STATUSBAR_VISIBLE KEY_UI "/statusbar_visible"
-#define TOOLBAR_VISIBLE KEY_UI "/toolbar_visible"
+static guint signal_id = 0;
+static gulong hook_id = 0;
 
-#define GNOME_INTERFACE "/desktop/gnome/interface"
-#define GNOME_TOOLBAR_STYLE GNOME_INTERFACE "/toolbar_style"
+/* Drag and trop target types */
+enum TargetTypes {
+    TEXT_PLAIN,
+    TEXT_URIS
+};
+
+/* Drag and drop targent entries */
+static const GtkTargetEntry target_entries[] = {
+   { "text/uri-list", 0, TEXT_URIS },
+   { "text/plain", 0, TEXT_PLAIN }
+};
 
 /* Quits seahorse */
 static void
 quit (GtkWidget *widget, SeahorseWidget *swidget)
 {
+	g_signal_remove_emission_hook (signal_id, hook_id);
 	seahorse_context_destroy (swidget->sctx);
-	gtk_exit (0);
 }
 
 /* Quits seahorse */
@@ -66,14 +73,115 @@ delete_event (GtkWidget *widget, GdkEvent *event, SeahorseWidget *swidget)
 static void
 generate_activate (GtkWidget *widget, SeahorseWidget *swidget)
 {
-	seahorse_generate_show (swidget->sctx);
+	seahorse_generate_select_show (swidget->sctx);
+}
+
+/* Setup our file types on a file chooser dialog */
+static void
+setup_file_types (GtkFileChooser* dialog)
+{
+    GtkFileFilter* filter = gtk_file_filter_new ();
+    gtk_file_filter_set_name (filter, _("All PGP key files"));
+    gtk_file_filter_add_pattern (filter, "*.asc");    
+    gtk_file_filter_add_pattern (filter, "*.key");    
+    gtk_file_filter_add_pattern (filter, "*.pkr");    
+    gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filter);    
+    gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (dialog), filter);
+
+    filter = gtk_file_filter_new ();
+    gtk_file_filter_set_name (filter, _("All files"));
+    gtk_file_filter_add_pattern (filter, "*");    
+    gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filter);    
 }
 
 /* Loads import dialog */
 static void
 import_activate (GtkWidget *widget, SeahorseWidget *swidget)
 {
-	seahorse_import_show (swidget->sctx);
+    GtkWidget *dialog;
+    char* uri = NULL;
+    gint keys;
+    gpgme_error_t err;
+    
+    dialog = gtk_file_chooser_dialog_new (_("Import Key"), 
+                GTK_WINDOW(glade_xml_get_widget (swidget->xml, "key-manager")),
+                GTK_FILE_CHOOSER_ACTION_OPEN, 
+                GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+                NULL);
+
+    setup_file_types (GTK_FILE_CHOOSER (dialog));
+    gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (dialog), FALSE);
+     
+    if(gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
+        uri = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (dialog));
+        
+    gtk_widget_destroy (dialog);
+
+    if(uri) {
+        keys = seahorse_op_import_file (swidget->sctx, uri, &err);
+        
+        if (GPG_IS_OK (err))
+            seahorse_context_keys_added (swidget->sctx, keys);
+        else
+            seahorse_util_handle_error (err, _("Couldn't import keys from \"%s\""), 
+                seahorse_util_uri_get_last (uri));
+                
+        g_free (uri);
+    }
+}
+
+/* Callback for pasting from clipboard */
+static void
+clipboard_received (GtkClipboard *board, const gchar *text, SeahorseContext *sctx)
+{
+    gpgme_error_t err;
+    gint keys;
+ 
+    keys = seahorse_op_import_text (sctx, text, &err);
+ 
+    if (!GPG_IS_OK (err))
+        seahorse_util_handle_error (err, _("Couldn't import keys from clipboard"));
+    else if (keys > 0)
+        seahorse_context_keys_added (sctx, keys);
+}
+
+/* Pastes key from keyboard */
+static void 
+paste_activate (GtkWidget *widget, SeahorseWidget *swidget)
+{
+    GdkAtom atom;
+    GtkClipboard *board;
+       
+    atom = gdk_atom_intern ("CLIPBOARD", FALSE);
+    board = gtk_clipboard_get (atom);
+    gtk_clipboard_request_text (board,
+         (GtkClipboardTextReceivedFunc)clipboard_received, swidget->sctx);
+}
+
+/* Copies key to clipboard */
+static void
+copy_activate (GtkWidget *widget, SeahorseWidget *swidget)
+{
+    GdkAtom atom;
+    GtkClipboard *board;
+    gchar *text;
+    gpgme_error_t err;
+    gpgme_key_t * recips;
+  
+    recips = seahorse_key_store_get_selected_recips (GTK_TREE_VIEW (
+        glade_xml_get_widget (swidget->xml, KEY_LIST)));
+       
+    text = seahorse_op_export_text (swidget->sctx, recips, &err);
+
+    if (!GPG_IS_OK (err))
+        seahorse_util_handle_error (err, _("Couldn't export key to clipboard"));
+    else if (text != NULL) {
+        atom = gdk_atom_intern ("CLIPBOARD", FALSE);
+        board = gtk_clipboard_get (atom);
+        gtk_clipboard_set_text (board, text, strlen (text));
+        g_free (text);
+    }
 }
 
 /* Loads key properties if a key is selected */
@@ -82,8 +190,9 @@ properties_activate (GtkWidget *widget, SeahorseWidget *swidget)
 {
 	SeahorseKey *skey;
 	
-	skey = seahorse_key_store_get_selected_key (GTK_TREE_VIEW (glade_xml_get_widget (swidget->xml, KEY_LIST)));
-	if (skey)
+	skey = seahorse_key_store_get_selected_key (GTK_TREE_VIEW (
+		glade_xml_get_widget (swidget->xml, KEY_LIST)));
+	if (skey != NULL)
 		seahorse_key_properties_new (swidget->sctx, skey);
 }
 
@@ -91,35 +200,89 @@ properties_activate (GtkWidget *widget, SeahorseWidget *swidget)
 static void
 export_activate (GtkWidget *widget, SeahorseWidget *swidget)
 {
-	SeahorseKey *skey;
-	
-	skey = seahorse_key_store_get_selected_key (GTK_TREE_VIEW (
-		glade_xml_get_widget (swidget->xml, KEY_LIST)));
-	if (skey)
-		seahorse_export_new (swidget->sctx, skey);
+    GtkWidget *dialog;
+    char* uri = NULL;
+    gpgme_error_t err;
+    gpgme_key_t* recips;
+    
+    dialog = gtk_file_chooser_dialog_new (_("Export Key"), 
+                GTK_WINDOW(glade_xml_get_widget (swidget->xml, "key-manager")),
+                GTK_FILE_CHOOSER_ACTION_SAVE, 
+                GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
+                NULL);
+
+    setup_file_types (GTK_FILE_CHOOSER(dialog));
+    gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (dialog), FALSE);
+     
+    while(gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT) {
+        GnomeVFSURI *u = gnome_vfs_uri_new (gtk_file_chooser_get_uri(GTK_FILE_CHOOSER(dialog)));
+        if (u != NULL) {
+            if (gnome_vfs_uri_exists(u)) {
+                GtkWidget* edlg;
+                edlg = gtk_message_dialog_new (GTK_WINDOW (dialog),
+                    GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_QUESTION,
+                    GTK_BUTTONS_NONE, _("A file already exists with this name.\n\nDo you want to overwrite it?"));
+                gtk_dialog_add_buttons (GTK_DIALOG (edlg), 
+                       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                       _("Replace"), GTK_RESPONSE_ACCEPT);
+
+                gtk_dialog_set_default_response (GTK_DIALOG (edlg), GTK_RESPONSE_CANCEL);                    
+                
+                if (gtk_dialog_run (GTK_DIALOG (edlg)) == GTK_RESPONSE_ACCEPT)
+                    uri = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (dialog));
+
+                gtk_widget_destroy (edlg);
+
+            /* File doesn't exist */
+            } else {
+                uri = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (dialog));
+            }
+            
+            gnome_vfs_uri_unref (u);
+
+        }
+
+        if (uri != NULL)
+            break;
+    }
+
+    gtk_widget_destroy (dialog);
+
+    if(uri) {
+        recips = seahorse_key_store_get_selected_recips (GTK_TREE_VIEW (
+            glade_xml_get_widget (swidget->xml, KEY_LIST)));
+
+        /* This frees recips */        
+        seahorse_op_export_file (swidget->sctx, uri, recips, &err); 
+        
+        if (!GPG_IS_OK (err))
+            seahorse_util_handle_error (err, _("Couldn't export key to \"%s\""),
+                    seahorse_util_uri_get_last (uri));
+                    
+        g_free (uri);
+    }
 }
 
 static void
 sign_activate (GtkWidget *widget, SeahorseWidget *swidget)
 {
-	SeahorseKey *skey;
+	GList *list = NULL;
 	
-	skey = seahorse_key_store_get_selected_key (GTK_TREE_VIEW (
+	list = seahorse_key_store_get_selected_keys (GTK_TREE_VIEW (
 		glade_xml_get_widget (swidget->xml, KEY_LIST)));
-	if (skey)
-		seahorse_sign_new (swidget->sctx, skey, 0);
+	seahorse_sign_show (swidget->sctx, list);
 }
 
 /* Loads delete dialog if a key is selected */
 static void
 delete_activate (GtkWidget *widget, SeahorseWidget *swidget)
 {
-	SeahorseKey *skey;
+	GList *list = NULL;
 	
-	skey = seahorse_key_store_get_selected_key (GTK_TREE_VIEW (
+	list = seahorse_key_store_get_selected_keys (GTK_TREE_VIEW (
 		glade_xml_get_widget (swidget->xml, KEY_LIST)));
-	if (skey)
-		seahorse_delete_new (swidget->sctx, skey, 0);
+	seahorse_delete_show (swidget->sctx, list);
 }
 
 static void
@@ -129,8 +292,8 @@ change_passphrase_activate (GtkMenuItem *item, SeahorseWidget *swidget)
 	
 	skey = seahorse_key_store_get_selected_key (GTK_TREE_VIEW (
 		glade_xml_get_widget (swidget->xml, KEY_LIST)));
-	if (skey)
-		seahorse_ops_key_change_pass (swidget->sctx, skey);
+	if (skey != NULL && SEAHORSE_IS_KEY_PAIR (skey))
+		seahorse_key_pair_op_change_pass (swidget->sctx, SEAHORSE_KEY_PAIR (skey));
 }
 
 static void
@@ -140,7 +303,7 @@ add_uid_activate (GtkMenuItem *item, SeahorseWidget *swidget)
 	
 	skey = seahorse_key_store_get_selected_key (GTK_TREE_VIEW (
 		glade_xml_get_widget (swidget->xml, KEY_LIST)));
-	if (skey)
+	if (skey != NULL)
 		seahorse_add_uid_new (swidget->sctx, skey);
 }
 
@@ -151,7 +314,7 @@ add_subkey_activate (GtkMenuItem *item, SeahorseWidget *swidget)
 	
 	skey = seahorse_key_store_get_selected_key (GTK_TREE_VIEW (
 		glade_xml_get_widget (swidget->xml, KEY_LIST)));
-	if (skey)
+	if (skey != NULL)
 		seahorse_add_subkey_new (swidget->sctx, skey);
 }
 
@@ -162,53 +325,10 @@ add_revoker_activate (GtkMenuItem *item, SeahorseWidget *swidget)
 	
 	skey = seahorse_key_store_get_selected_key (GTK_TREE_VIEW (
 		glade_xml_get_widget (swidget->xml, KEY_LIST)));
-	if (skey)
+	if (skey != NULL)
 		seahorse_add_revoker_new (swidget->sctx, skey);
 }
 
-static void
-encrypt_file_activate (GtkMenuItem *item, SeahorseWidget *swidget)
-{
-	seahorse_encrypt_file_new (swidget->sctx);
-}
-
-static void
-sign_file_activate (GtkMenuItem *item, SeahorseWidget *swidget)
-{
-	seahorse_sign_file_new (swidget->sctx);
-}
-
-static void
-encrypt_sign_file_activate (GtkMenuItem *item, SeahorseWidget *swidget)
-{
-	seahorse_encrypt_sign_file_new (swidget->sctx);
-}
-
-static void
-decrypt_file_activate (GtkMenuItem *item, SeahorseWidget *swidget)
-{
-	seahorse_decrypt_file_new (swidget->sctx);
-}
-
-static void
-verify_file_activate (GtkMenuItem *item, SeahorseWidget *swidget)
-{
-	seahorse_verify_file_new (swidget->sctx);
-}
-
-static void
-decrypt_verify_file_activate (GtkMenuItem *item, SeahorseWidget *swidget)
-{
-	seahorse_decrypt_verify_file_new (swidget->sctx);
-}
-
-/* Loads text editor */
-static void
-text_editor_activate (GtkMenuItem *widget, SeahorseWidget *swidget)
-{
-	seahorse_text_editor_show (swidget->sctx);
-}
-
 /* Loads preferences dialog */
 static void
 preferences_activate (GtkWidget *widget, SeahorseWidget *swidget)
@@ -216,12 +336,6 @@ preferences_activate (GtkWidget *widget, SeahorseWidget *swidget)
 	seahorse_preferences_show (swidget->sctx);
 }
 
-static void
-view_bar (GtkCheckMenuItem *item, const gchar *key)
-{
-	eel_gconf_set_boolean (key, gtk_check_menu_item_get_active (item));
-}
-
 static void
 expand_all_activate (GtkMenuItem *item, SeahorseWidget *swidget)
 {
@@ -245,6 +359,7 @@ about_activate (GtkWidget *widget, SeahorseWidget *swidget)
 		"Jacob Perkins <jap1@users.sourceforge.net>",
 		"Jose Carlos Garcia Sogo <jsogo@users.sourceforge.net>",
 		"Jean Schurger <yshark@schurger.org>",
+        "Nate Nielsen <nielsen@memberwebs.com>",
 		NULL
 	};
 
@@ -304,37 +419,65 @@ row_activated (GtkTreeView *treeview, GtkTreePath *path, GtkTreeViewColumn *arg2
 }
 
 static void
-selection_changed (GtkTreeSelection *selection, SeahorseWidget *swidget)
+set_key_options_sensitive (SeahorseWidget *swidget, gboolean selected, gboolean secret, SeahorseKey *skey)
 {
-	gboolean sensitive = FALSE,
-		 secret = FALSE;
-	SeahorseKey *skey = NULL;
+	gboolean create = FALSE;
 	
-	sensitive = gtk_tree_selection_get_selected (selection, NULL, NULL);
-	if (sensitive) {
-		skey = seahorse_key_store_get_selected_key (GTK_TREE_VIEW (
-			glade_xml_get_widget (swidget->xml, KEY_LIST)));
-		secret = SEAHORSE_IS_KEY_PAIR (skey);
-	}
+	/* items that can do multiple */;
+	create = (selected && seahorse_key_widget_can_create ("sign", skey));
+	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "sign_button"), create);
+	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "sign"), create);
+	
+	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "delete_button"), selected);
+	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "delete"), selected);
+	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "export_button"), selected);
+	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "export"), selected);
+    gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "copy_key"), selected);
+	
+	/* items that can do single */
+	create = (skey != NULL && seahorse_key_widget_can_create ("key-properties", skey));
+	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "properties"), create);
+	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "properties_button"), create);
+	
+	/* items that need a secret key */
+	create = (secret && seahorse_key_widget_can_create ("add-uid", skey));
+	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "add_uid"), create);
+	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "key_add_uid"), create);
+	
+	create = (secret && seahorse_key_widget_can_create ("add-subkey", skey));
+	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "add_subkey"), create);
+	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "key_add_subkey"), create);
 	
-	/* do all key operation items */
-	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "properties_button"), sensitive);
-	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "export_button"), sensitive);
-	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "sign_button"), sensitive);
-	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "delete_button"), sensitive);
-	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "properties"), sensitive);
-	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "export"), sensitive);
-	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "sign"), sensitive);
-	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "delete"), sensitive);
-	/* ops requiring a secret key */
-	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "change_passphrase"), secret);
-	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "add_uid"), secret);
-	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "add_subkey"), secret);
 	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "add_revoker"), secret);
 	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "key_change_passphrase"), secret);
-	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "key_add_uid"), secret);
-	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "key_add_subkey"), secret);
 	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "key_add_revoker"), secret);
+	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "change_passphrase"), secret);
+}
+
+static void
+selection_changed (GtkTreeSelection *selection, SeahorseWidget *swidget)
+{
+	gint rows = 0;
+	gboolean selected = FALSE, secret = FALSE;
+	SeahorseKey *skey = NULL;
+	
+	rows = gtk_tree_selection_count_selected_rows (selection);
+	selected = rows > 0;
+	
+	if (selected) {
+		GnomeAppBar *status;
+		
+		status = GNOME_APPBAR (glade_xml_get_widget (swidget->xml, "status"));
+		gnome_appbar_set_status (status, g_strdup_printf ("Selected %d keys", rows));
+	}
+	
+	if (rows == 1) {
+		skey = seahorse_key_store_get_selected_key (GTK_TREE_VIEW (
+			glade_xml_get_widget (swidget->xml, KEY_LIST)));
+		secret = (skey != NULL && SEAHORSE_IS_KEY_PAIR (skey));
+	}
+	
+	set_key_options_sensitive (swidget, selected, secret, skey);
 }
 
 static void
@@ -362,67 +505,29 @@ key_list_popup_menu (GtkWidget *widget, SeahorseWidget *swidget)
 	SeahorseKey *skey;
 	
 	skey = seahorse_key_store_get_selected_key (GTK_TREE_VIEW (widget));
-	if (skey)
+	if (skey != NULL)
 		show_context_menu (swidget, 0, gtk_get_current_event_time ());
+       
+    return FALSE;
 }
 
-static void
-set_toolbar_style (GtkToolbar *toolbar, const gchar *style)
-{
-	if (g_str_equal (style, TOOLBAR_BOTH))
-		gtk_toolbar_set_style (toolbar, GTK_TOOLBAR_BOTH);
-	else if (g_str_equal (style, TOOLBAR_BOTH_HORIZ))
-		gtk_toolbar_set_style (toolbar, GTK_TOOLBAR_BOTH_HORIZ);
-	else if (g_str_equal (style, TOOLBAR_TEXT))
-		gtk_toolbar_set_style (toolbar, GTK_TOOLBAR_TEXT);
-	else if (g_str_equal (style, TOOLBAR_ICONS))
-		gtk_toolbar_set_style (toolbar, GTK_TOOLBAR_ICONS);
-}
 
 static void
 gconf_notification (GConfClient *gclient, guint id, GConfEntry *entry, SeahorseWidget *swidget)
 {
 	const gchar *key;
 	GConfValue *value;
-	GtkWidget *widget;
 	
 	key = gconf_entry_get_key (entry);
 	value = gconf_entry_get_value (entry);
-	
-	if (g_str_equal (key, STATUSBAR_VISIBLE)) {
-		widget = glade_xml_get_widget (swidget->xml, "status");
-		
-		if (gconf_value_get_bool (value))
-			gtk_widget_show (widget);
-		else
-			gtk_widget_hide (widget);
-	}
-	else if (g_str_equal (key, TOOLBAR_VISIBLE)) {
-		widget = glade_xml_get_widget (swidget->xml, "tool_dock");
-		
-		if (gconf_value_get_bool (value))
-			gtk_widget_show (widget);
-		else
-			gtk_widget_hide (widget);
-	}
-	else if (g_str_equal (key, GNOME_TOOLBAR_STYLE) &&
-	g_str_equal (TOOLBAR_DEFAULT, eel_gconf_get_string (KEY_TOOLBAR_STYLE))) {
-		set_toolbar_style (GTK_TOOLBAR (glade_xml_get_widget (swidget->xml, "toolbar")),
-			gconf_value_get_string (value));
-	}
-	else if (g_str_equal (key, KEY_TOOLBAR_STYLE)) {
-		widget = glade_xml_get_widget (swidget->xml, "toolbar");
-		
-		/* if changed to default, use system settings */
-		if (g_str_equal (gconf_value_get_string (value), TOOLBAR_DEFAULT)) {
-			set_toolbar_style (GTK_TOOLBAR (widget),
-				eel_gconf_get_string (GNOME_TOOLBAR_STYLE));
-		}
-		else
-			set_toolbar_style (GTK_TOOLBAR (widget), gconf_value_get_string (value));
-	}
+
+    /* 
+     * Removed last two gconf notification items, although I'm sure
+     * we'll get more, so leaving this code here... 
+     */
 }
 
+#if 0
 static void
 show_progress (SeahorseContext *sctx, const gchar *op, gdouble fract, SeahorseWidget *swidget)
 {
@@ -448,7 +553,6 @@ show_progress (SeahorseContext *sctx, const gchar *op, gdouble fract, SeahorseWi
 	
 	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "key"), sensitive);
 	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "edit"), sensitive);
-	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "tools"), sensitive);
 	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "properties_button"), sensitive);
 	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "export_button"), sensitive);
 	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "sign_button"), sensitive);
@@ -458,22 +562,113 @@ show_progress (SeahorseContext *sctx, const gchar *op, gdouble fract, SeahorseWi
 	while (g_main_context_pending (NULL))
 		g_main_context_iteration (NULL, TRUE);
 }
+#endif
 
-void
+/* params[0] = sctx, params[1] = op string, params[2] = fract */
+static gboolean
+progress_hook (GSignalInvocationHint *hint, guint n_params, const GValue *params, SeahorseWidget *swidget)
+{
+	GnomeAppBar *status;
+	GtkProgressBar *progress;
+	/* gboolean sensitive; */
+	gdouble fract;
+
+	fract = g_value_get_double (&params[2]);
+	/* sensitive = (fract == -1); */
+	
+	status = GNOME_APPBAR (glade_xml_get_widget (swidget->xml, "status"));
+	gnome_appbar_set_status (status, g_value_get_string (&params[1]));
+	progress = gnome_appbar_get_progress (status);
+	/* do progress */
+	if (fract <= 1 && fract > 0)
+		gtk_progress_bar_set_fraction (progress, fract);
+	else if (fract != -1) {
+		gtk_progress_bar_set_pulse_step (progress, 0.05);
+		gtk_progress_bar_pulse (progress);
+	}
+	/* if fract == -1, cleanup progress */
+	else
+		gtk_progress_bar_set_fraction (progress, 0);
+	
+	while (g_main_context_pending (NULL))
+		g_main_context_iteration (NULL, TRUE);
+	
+	return TRUE;
+}
+
+static void
+target_drag_data_received (GtkWidget *widget, GdkDragContext *context, gint x, gint y,
+                           GtkSelectionData *data, guint info, guint time, SeahorseContext *sctx)
+{
+    gint keys = 0;
+    gpgme_error_t err;
+    gchar** uris;
+    gchar** u;
+    
+    g_return_if_fail (data != NULL);
+    
+    switch(info) {
+    case TEXT_PLAIN:
+        keys = seahorse_op_import_text (sctx, data->data, &err);
+        break;
+        
+    case TEXT_URIS:
+        uris = g_strsplit (data->data, "\n", 0);
+        for(u = uris; *u; u++) {
+            g_strstrip (*u);
+            if ((*u)[0]) { /* Make sure it's not an empty line */
+                keys += seahorse_op_import_file (sctx, *u, &err);  
+                if (!GPG_IS_OK (err)) {
+                    seahorse_util_handle_error (err, _("Couldn't import key from \"%s\""),
+                            seahorse_util_uri_get_last (*u));
+                    break;
+                }
+            }
+        }
+        g_strfreev (uris);
+        break;
+        
+    default:
+        g_assert_not_reached ();
+        return;
+    } 
+
+    if (keys > 0)
+        seahorse_context_keys_added (sctx, keys);        
+}
+
+/* Refilter the keys when the filter text changes */
+static void
+filter_changed (GtkWidget *widget, SeahorseKeyStore* skstore)
+{
+    const gchar* text = gtk_entry_get_text (GTK_ENTRY (widget));
+    g_object_set (skstore, "filter", text, NULL);
+}
+
+GtkWindow* 
 seahorse_key_manager_show (SeahorseContext *sctx)
 {
 	SeahorseWidget *swidget;
 	GtkTreeView *view;
+    GtkWidget* w;
 	GtkTreeSelection *selection;
-	GtkWidget *widget;
-	gboolean visible;
+    SeahorseKeyStore *skstore;
 	
 	swidget = seahorse_widget_new ("key-manager", sctx);
 	gtk_object_sink (GTK_OBJECT (sctx));
 	
 	/* construct key context menu */
-	glade_xml_construct (swidget->xml, SEAHORSE_GLADEDIR "seahorse-key-manager.glade2",
+	glade_xml_construct (swidget->xml, SEAHORSE_GLADEDIR "seahorse-key-manager.glade",
 		"context_menu", NULL);
+	set_key_options_sensitive (swidget, FALSE, FALSE, NULL);
+	
+	//features not available
+	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "add_photo"), FALSE);
+	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "key_add_photo"), FALSE);
+	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "backup"), FALSE);
+	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "key_backup"), FALSE);
+	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "gen_revoke"), FALSE);
+	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "key_gen_revoke"), FALSE);
 	
 	/* quit signals */
 	glade_xml_signal_connect_data (swidget->xml, "quit",
@@ -481,68 +676,17 @@ seahorse_key_manager_show (SeahorseContext *sctx)
 	glade_xml_signal_connect_data (swidget->xml, "quit_event",
 		G_CALLBACK (delete_event), swidget);
 	
-	g_signal_connect (swidget->sctx, "progress", G_CALLBACK (show_progress), swidget);
-	
-	/* init gclient */
-	eel_gconf_notification_add (KEY_UI, (GConfClientNotifyFunc) gconf_notification, swidget);
-	eel_gconf_monitor_add (KEY_UI);
-	eel_gconf_notification_add (GNOME_INTERFACE, (GConfClientNotifyFunc) gconf_notification, swidget);
-	eel_gconf_monitor_add (GNOME_INTERFACE);
-	
-	/* init toolbar */
-	glade_xml_signal_connect_data (swidget->xml, "toolbar_activate",
-		G_CALLBACK (view_bar), TOOLBAR_VISIBLE);
-	set_toolbar_style (GTK_TOOLBAR (glade_xml_get_widget (swidget->xml, "toolbar")),
-		eel_gconf_get_string (KEY_TOOLBAR_STYLE));
-	visible = eel_gconf_get_boolean (TOOLBAR_VISIBLE);
-	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (
-		glade_xml_get_widget (swidget->xml, "view_toolbar")), visible);
-	if (!visible)
-		gtk_widget_hide (glade_xml_get_widget (swidget->xml, "tool_dock"));
-	
-	/* init status bar */
-	widget = glade_xml_get_widget (swidget->xml, "view_statusbar");
-	visible = eel_gconf_get_boolean (STATUSBAR_VISIBLE);
-	glade_xml_signal_connect_data (swidget->xml, "statusbar_activate",
-		G_CALLBACK (view_bar), STATUSBAR_VISIBLE);
-	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (widget), visible);
-	if (!visible)
-		gtk_widget_hide (glade_xml_get_widget (swidget->xml, "status"));
-	
-	/* other signals */	
-	glade_xml_signal_connect_data (swidget->xml, "preferences_activate",
-		G_CALLBACK (preferences_activate), swidget);
-	glade_xml_signal_connect_data (swidget->xml, "about_activate",
-		G_CALLBACK (about_activate), swidget);
-	
-	/* init key list & selection settings */
-	view = GTK_TREE_VIEW (glade_xml_get_widget (swidget->xml, KEY_LIST));
-	selection = gtk_tree_view_get_selection (view);
-	g_signal_connect (selection, "changed",
-		G_CALLBACK (selection_changed), swidget);
-	seahorse_key_manager_store_new (sctx, view);
-	selection_changed (selection, swidget);
-	
 	/* key menu signals */
 	glade_xml_signal_connect_data (swidget->xml, "generate_activate",
 		G_CALLBACK (generate_activate), swidget);
 	glade_xml_signal_connect_data (swidget->xml, "import_activate",
-		G_CALLBACK (import_activate), swidget);	
-	/* tool menu signals */
-	glade_xml_signal_connect_data (swidget->xml, "encrypt_file_activate",
-		G_CALLBACK (encrypt_file_activate), swidget);
-	glade_xml_signal_connect_data (swidget->xml, "sign_file_activate",
-		G_CALLBACK (sign_file_activate), swidget);
-	glade_xml_signal_connect_data (swidget->xml, "encrypt_sign_file_activate",
-		G_CALLBACK (encrypt_sign_file_activate), swidget);
-	glade_xml_signal_connect_data (swidget->xml, "decrypt_file_activate",
-		G_CALLBACK (decrypt_file_activate), swidget);
-	glade_xml_signal_connect_data (swidget->xml, "verify_file_activate",
-		G_CALLBACK (verify_file_activate), swidget);
-	glade_xml_signal_connect_data (swidget->xml, "decrypt_verify_file_activate",
-		G_CALLBACK (decrypt_verify_file_activate), swidget);
-	glade_xml_signal_connect_data (swidget->xml, "text_editor_activate",
-		G_CALLBACK (text_editor_activate), swidget);	
+		G_CALLBACK (import_activate), swidget);
+	
+	glade_xml_signal_connect_data (swidget->xml, "expand_all_activate",
+		G_CALLBACK (expand_all_activate), swidget);
+	glade_xml_signal_connect_data (swidget->xml, "collapse_all_activate",
+		G_CALLBACK (collapse_all_activate), swidget);
+		
 	/* tree view signals */	
 	glade_xml_signal_connect_data (swidget->xml, "row_activated",
 		G_CALLBACK (row_activated), swidget);
@@ -559,6 +703,8 @@ seahorse_key_manager_show (SeahorseContext *sctx)
 		G_CALLBACK (sign_activate), swidget);
 	glade_xml_signal_connect_data (swidget->xml, "delete_activate",
 		G_CALLBACK (delete_activate), swidget);
+    glade_xml_signal_connect_data (swidget->xml, "copy_activate",
+        G_CALLBACK (copy_activate), swidget);      
 	/* selected key with secret signals */
 	glade_xml_signal_connect_data (swidget->xml, "change_passphrase_activate",
 		G_CALLBACK (change_passphrase_activate), swidget);
@@ -569,16 +715,44 @@ seahorse_key_manager_show (SeahorseContext *sctx)
 	glade_xml_signal_connect_data (swidget->xml, "add_revoker_activate",
 		G_CALLBACK (add_revoker_activate), swidget);
 	
-	glade_xml_signal_connect_data (swidget->xml, "expand_all_activate",
-		G_CALLBACK (expand_all_activate), swidget);
-	glade_xml_signal_connect_data (swidget->xml, "collapse_all_activate",
-		G_CALLBACK (collapse_all_activate), swidget);
+	//g_signal_connect (swidget->sctx, "progress", G_CALLBACK (show_progress), swidget);
+	signal_id = g_signal_lookup ("progress", G_OBJECT_CLASS_TYPE (G_OBJECT_CLASS (SEAHORSE_CONTEXT_GET_CLASS (sctx))));
+	hook_id = g_signal_add_emission_hook (signal_id, 0, (GSignalEmissionHook)progress_hook,
+		swidget, (GDestroyNotify)seahorse_widget_destroy);
 	
-	//features not available
-	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "add_photo"), FALSE);
-	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "key_add_photo"), FALSE);
-	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "backup"), FALSE);
-	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "key_backup"), FALSE);
-	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "gen_revoke"), FALSE);
-	gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "key_gen_revoke"), FALSE);
+	/* init gclient */
+	eel_gconf_notification_add (UI_SCHEMAS, (GConfClientNotifyFunc) gconf_notification, swidget);
+	eel_gconf_monitor_add (UI_SCHEMAS);
+	
+	/* other signals */	
+	glade_xml_signal_connect_data (swidget->xml, "preferences_activate",
+		G_CALLBACK (preferences_activate), swidget);
+    glade_xml_signal_connect_data (swidget->xml, "paste_activate",
+       G_CALLBACK (paste_activate), swidget);
+	glade_xml_signal_connect_data (swidget->xml, "about_activate",
+		G_CALLBACK (about_activate), swidget);
+	
+	/* init key list & selection settings */
+	view = GTK_TREE_VIEW (glade_xml_get_widget (swidget->xml, KEY_LIST));
+	selection = gtk_tree_view_get_selection (view);
+	gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE);
+	g_signal_connect (selection, "changed",
+		G_CALLBACK (selection_changed), swidget);
+    skstore = seahorse_key_manager_store_new (sctx, view);
+	selection_changed (selection, swidget);
+   
+    w = glade_xml_get_widget (swidget->xml, "key-manager");
+    
+    /* Setup drops */
+    gtk_drag_dest_set (w, GTK_DEST_DEFAULT_ALL, target_entries, 
+            sizeof (target_entries) / sizeof (target_entries[0]), GDK_ACTION_COPY);
+    gtk_signal_connect (GTK_OBJECT (w), "drag_data_received",
+            GTK_SIGNAL_FUNC (target_drag_data_received), sctx);
+                        
+    /* For the filtering */
+    glade_xml_signal_connect_data(swidget->xml, "on_filter_changed",
+                              G_CALLBACK(filter_changed), skstore);
+
+    seahorse_context_get_keys (sctx);                                 
+    return GTK_WINDOW (w);
 }
