@@ -26,14 +26,13 @@
 
 #include "seahorse-context.h"
 
-void	seahorse_key_manager_show	(SeahorseContext	*sctx);
+GtkWindow*	seahorse_key_manager_show	(SeahorseContext	*sctx);
 
-void	seahorse_generate_show		(SeahorseContext	*sctx);
+void	seahorse_generate_select_show	(SeahorseContext	*sctx);
 
-void	seahorse_import_show		(SeahorseContext	*sctx);
+void	seahorse_generate_adv_show	(SeahorseContext	*sctx);
 
-void	seahorse_export_show		(SeahorseContext	*sctx,
-					 GpgmeRecipients	recips);
+void	seahorse_generate_druid_show	(SeahorseContext	*sctx);
 
 void	seahorse_delete_show		(SeahorseContext	*sctx,
 					 GList			*keys);
@@ -41,4 +40,8 @@ void	seahorse_delete_show		(SeahorseContext	*sctx,
 void	seahorse_sign_show		(SeahorseContext	*sctx,
 					 GList			*keys);
 
+gchar** seahorse_process_multiple (SeahorseContext       *sctx, 
+                                   const gchar**         uris, 
+                                   const gchar*          glade);
+
 #endif /* __SEAHORSE_WINDOWS_H__ */
